
from ScrumPy.Data import DataSets # results will be returned in a DataSet so we must import this

import BuildLP # this module is specific to this exercise and can be found in ../Model/Tools



def ATPScan(m,Alo,Ahi,nPoints, lp=None):
    '''
        pre : m      -  the model 
              lo      - lower limit for ATPase flux 
              hi      - upper limit for ATPase flux
              nPoints - number of points to sample
              lp      - an lp object, if None a new one will be generated
                
       post:  returns a dataset (ds) containing the scan results. ds.lp will be the lp object
    '''

    
    rv = DataSets.DataSet() # where our results will be stored
                            # rv stands for "Return Value"

    
    if lp == None:              # if the lp object is equal to None, build a new one
        lp = BuildLP.BuildLP(m)
    
    ATPase = Alo                    # lower value for the ATPase flux
    incr = (Ahi-Alo)/(nPoints-1.0)  # the step size for the ATPase flux from the range and number of points
    
    for n in range(nPoints):                # for each step
        lp.SetFixedFlux({'ATPase':ATPase})  # set the "ATPase" flux in the lp to the current ATPase value
        lp.Solve()                          # solve the lp
        rv.UpdateFromDic(lp.GetPrimSol())   # update the returned data-set with the current solution
        ATPase += incr                      # update the current value of ATPase

    rv.MakeFirstCol("ATPase")   # make the first column ATPase - default X axis for plotting
    rv.lp = lp                  # add the lp to the datset - make it available to the user

    return rv # return the results to the user - don't forget to do this !


def GetChangers(ds, tol=1e-6):
    """ pre: ds = ATPScan(...)
       post: GetChangers(ds) => list of reactions in ds whose flux changes by more than tol
    """

    rv = []

    for c in ds.cnames[1:]:  # ignore first col, it's the ATPase
        if abs(ds.MaxInCol(c) - ds.MinInCol(c)) > tol:
            rv.append(c)

    return rv

def GetSwitchers(ds):
    """ pre: ds = ATPScan(...)
       post: GetSwitchers(ds) => list of reactions that have at least one value of zero
    """
    
    rv = []

    for c in ds.cnames[1:]:  # ignore first col, it's the ATPase
        if min(map(abs, ds.GetCol(c))) == 0.0:
            rv.append(c)

    return rv


def GetRanges(ds, tol=1e-6):
    """ pre: ds = ATPScan(...)
       post: GetRanges(ds) => dictionary mapping reactions -> max - min flux
    """
    rv = {}

    for c in ds.cnames[1:]:
        fluxrange = abs( ds.MaxInCol(c) - ds.MinInCol(c))
        if fluxrange > tol:
            rv[c] = fluxrange

    return rv
    

    
